'use strict';

var _ = require('lodash');
var debug = require('debug')('generator-ats-node-webapp:app');

module.exports = {
  packageJson: function packageJson() {
    this.appendPackageJson({
      name: _.kebabCase(this.props.name),
      version: this.props.version,
      description: this.props.description,
      homepage: this.props.homepage,
      scripts: {},
      repository: {
        type: 'git',
        url: 'https://github.com/' + this.props.repository
      },
      bugs: {
        url: 'https://github.com/' + this.props.repository + '/issues'
      },
      author: {
        name: this.props.authorName,
        email: this.props.authorEmail,
        url: this.props.authorUrl
      },
      files: ['client'],
      keywords: this.props.keywords
    });
  },

  writeTemplates: function writeTemplates() {
    // Do Nothing
  },

  subgenerators: function subgenerators() {
    var _this = this;

    var composeLocal = function composeLocal(name) {
      debug('composing with generator', name);
      _this.composeWith('ats-node-webapp:' + name, { options: _this.props }, { local: require.resolve('../../' + name) });
    };
    var useOAuthStrategy = function useOAuthStrategy(name) {
      return (_this.props.oauthStrategies || []).indexOf(name) > -1;
    };
    var useDatabase = function useDatabase(name) {
      return _this.props.database === name;
    };
    var composeOAuth = function composeOAuth(name) {
      if (useOAuthStrategy(name)) {
        composeLocal('auth-' + name);
      }
    };

    this.props.useOAuthStrategy = useOAuthStrategy;
    this.props.useDatabase = useDatabase;
    debug('generating application with properties', this.props);

    composeLocal('skeleton');
    composeLocal('server-' + this.props.server);
    if (this.props.server !== 'thin') {
      composeLocal('app-symlink');
    }

    // Add Server Components
    if (this.props.server === 'full') {
      // Add OAuth Generators
      composeOAuth('facebook');
      composeOAuth('github');
      composeOAuth('google');
      composeOAuth('twitter');
      composeOAuth('linkedin');

      // Add Persistence
      composeLocal('persistence-' + this.props.database);
    }

    // Add Client
    composeLocal('client-' + this.props.client);
  }
};